#include "StdAfx.h"
#include "Model.h"

// DEBUG_NEW macro allows MFC applications to determine memory leak locations in debug builds
#ifdef _DEBUG
  #define new DEBUG_NEW
#endif

CModel::CModel()
: m_root_node( L"root node" )
, m_PointLight(0)
, m_DirectionalLight(1)
, m_SpotLight(2)
{  
  UWB_SceneNode* pFloorNode = new UWB_SceneNode(L"CustomMesh");
  m_CustomMesh = new UWBD3D_PrimitiveMeshCustom(30, L"VeryHiResMesh.x");
  pFloorNode->SetPrimitive(m_CustomMesh);
  UWB_XformInfo floorXf = pFloorNode->GetXformInfo();
  floorXf.SetScale(vec3(15.0f, 15.0f, 15.0f));
  pFloorNode->SetXformInfo(floorXf);
  m_root_node.InsertChildNode( pFloorNode );
  
  UWB_SceneNode *pCraftNode = new UWB_SceneNode(L"shusui");
  UWBD3D_PrimitiveMesh *pCraft = new UWBD3D_PrimitiveMesh(L"shusui.x");
  UWB_XformInfo sxf = pCraftNode->GetXformInfo();
  sxf.SetTranslation(vec3(3, 5, -3));
  pCraftNode->SetXformInfo(sxf);
  pCraftNode->SetPrimitive(pCraft);
  m_root_node.InsertChildNode(pCraftNode);

  UWB_SceneNode *pRecNode = new UWB_SceneNode(L"rectangle");
  UWB_PrimitiveRectangleXZ *pR = new UWB_PrimitiveRectangleXZ();
  pR->SetCorners(vec3(1.0f, 0.0f, 1.0f), vec3(-1.0f, 0.0f, -1.0f));
  UWB_XformInfo rxf = pRecNode->GetXformInfo();
  rxf.SetTranslation(vec3(-8.0f, 3.0f, 5.0f));
  rxf.SetScale(vec3(2.0f, 1.0f, 2.0f));
  pRecNode->SetXformInfo(rxf);
  UWB_Material m = pR->GetMaterial();
  m.SetDiffuse(0.8f);
  m.SetAmbient(1.0f, 0.8f, 0.8f);
  pR->SetMaterial(m);
  pRecNode->SetPrimitive(pR);
  m_root_node.InsertChildNode(pRecNode);

  
  // Lighting condition ...
  m_PointLight.SwitchOnLight();
  m_PointLight.SetPosition(2.0f, 12.0f, 2.0f);
  m_PointLight.SetDiffuse(0.2f);

  m_DirectionalLight.SwitchOnLight();
  m_DirectionalLight.SetPosition(0.0f, 12.0f, 0.0f);
  m_DirectionalLight.SetDiffuse(0.2f, 0.1f, 0.1f);

  m_SpotLight.SwitchOnLight();
  m_SpotLight.SetPosition(-2.0f, 12.0f, 2.0f);
  m_SpotLight.SetDiffuse(0.2f, 0.2f, 0.5f);
}

void CModel::DrawModel()
{
    UWBD3D_DrawHelper draw_helper;  

    m_PointLight.SetLightToDevice();
		m_DirectionalLight.SetLightToDevice();
		m_SpotLight.SetLightToDevice();
				 
		m_DirectionalLight.DrawLight(draw_helper);
		m_SpotLight.DrawLight(draw_helper);
		m_PointLight.DrawLight(draw_helper);

  
    m_root_node.Draw( draw_helper.GetLOD(), draw_helper );
}

UWB_SceneNode* CModel::GetRootNode()
{
  return &m_root_node;
}