#include "uwbgl_D3DEffectResource1.h"

UWBD3D_EffectResource::UWBD3D_EffectResource()
: m_pEffect( NULL )
{
}

UWBD3D_EffectResource::~UWBD3D_EffectResource() 
{
  ReleaseMemory();
}

void UWBD3D_EffectResource::ReleaseMemory()
{
  __super::ReleaseMemory();
  
  if (NULL != m_pEffect)
      m_pEffect->Release();
  m_pEffect = NULL;
}


// Load the mesh (in D3D .x format) and store into the buffer of the device
bool UWBD3D_EffectResource::LoadResourceFromFile(const wchar_t* fullPathName)
{
    LPD3DXBUFFER errorBuf;

    // Load the effect during inintialization
    // DWORD dwShaderFlags = D3DXFX_NOT_CLONEABLE;
    DWORD dwShaderFlags = 0L;

#if defined( DEBUG ) || defined( _DEBUG )
    // Set the D3DXSHADER_DEBUG flag to embed debug information in the shaders.
    // Setting this flag improves the shader debugging experience, but still allows 
    // the shaders to be optimized and to run exactly the way they will run in 
    // the release configuration of this program.
    dwShaderFlags |= D3DXSHADER_DEBUG;
#endif
     
    HRESULT stat = D3DXCreateEffectFromFile(m_pDevice, fullPathName, 
                       NULL, NULL, dwShaderFlags, NULL, &m_pEffect, &errorBuf);
    if (D3D_OK != stat) {
        std::wstring err(L"Unknown");
        switch (stat) {
           case D3DERR_INVALIDCALL:
               err = L"Invalid Call";
               break;
           case D3DXERR_INVALIDDATA:
               err = L"Invalid Data";
               break;
           case E_OUTOFMEMORY:
               err = L"Out of Memory";
        }
        char *msg = NULL;
        if (NULL != errorBuf) {
            msg = (char *) errorBuf->GetBufferPointer();
        }

       MessageBox(NULL, L"Effect Loading Failed!", err.c_str(), MB_OK);
       return false;
    }
    return true;
}