#include "uwbgl_D3DGraphicsSystem7.h"


struct PixelMaterialStruct {
        float Ambient[4],
              Diffuse[4],
              Specular[4],
              Emissive[4],
              Shininess;
};

void TranslateMaterial(PixelMaterialStruct &pm, UWB_Material &m)
{
#define GetChannel(c,a,b) pm.##c##[a] = m.Get##c####b##();
#define GetType(c) {\
        GetChannel(c, 0, R)\
        GetChannel(c, 1, G)\
        GetChannel(c, 2, B)\
        GetChannel(c, 3, A)\
}     
    GetType(Ambient)
    GetType(Diffuse)
    GetType(Specular)
    GetType(Emissive)
    pm.Shininess = m.GetPower();

}

void UWBD3D_GraphicsSystem::SetEffectParameters(UWB_Primitive& aPrimitive)
{
    D3DXMATRIX world, view, projection;
    m_pD3DDevice->GetTransform(D3DTS_WORLD, &world);
    m_pD3DDevice->GetTransform(D3DTS_VIEW, &view);
    m_pD3DDevice->GetTransform(D3DTS_PROJECTION, &projection);

    D3DXMATRIX worldI, worldIT;
    D3DXMatrixInverse(&worldI, NULL, &world);
    D3DXMatrixTranspose(&worldIT, &worldI);


    m_pCurrentEffect->SetMatrix("world", &world);
    m_pCurrentEffect->SetMatrix("view", &view);
    m_pCurrentEffect->SetMatrix("projection", &projection);
    m_pCurrentEffect->SetMatrix("worldIT", &worldIT);


    PixelMaterialStruct pm;
    UWB_Material material = aPrimitive.GetMaterial();
    TranslateMaterial(pm, material);
    
    m_pCurrentEffect->SetValue("material", &pm, sizeof(pm));

    m_pCurrentEffect->SetBool("textureEnable", aPrimitive.IsTexturingEnabled());
    if (aPrimitive.IsTexturingEnabled() ) {
        UWBD3D_TextureResource* pTexture = FindTextureResource( aPrimitive.GetTextureFileName(), true );
        if (NULL != pTexture) 
           m_pCurrentEffect->SetTexture("textureMap", pTexture->GetD3DTexture());
    } else {
        m_pCurrentEffect->SetTexture("textureMap", NULL);
    }

    // m_pCurrentEffect->SetTechnique("MultipleLights");
    D3DXHANDLE tech;
    m_pCurrentEffect->FindNextValidTechnique(NULL, &tech);
    m_pCurrentEffect->SetTechnique(tech);
}

bool UWBD3D_GraphicsSystem::SetCurrentEffect(LPCTSTR effectName) 
{
    UWBD3D_EffectResource *pEffect = FindEffectResource(effectName, true);
    if (NULL != pEffect) {
        m_pCurrentEffect = pEffect->GetD3DEffct();
        return true;
    } 
    return false;
}