#include "uwbgl_D3DGraphicsSystem7.h"

// items in the texture and mesh tables are sorted by name. This is a binary
// search function that finds a resource in the list very quickly. Typically these samples
// do not contain a lot of textures, so the speed improvements of a binary search over linear
// search is minimal, but it is always best to perform a binary search if the data is sorted
template<class T>
static T* BinarySearchResources( const wchar_t* name, UWB_PointerArray<T*>& table, int span_min, int span_max )
{
  if( span_max < span_min )
    return NULL;

  int index = (span_min + span_max) / 2;
  if( index < 0 || index >= table.Count() )
    return NULL;

  T* pResource = table[index];
  int compare_result = ::wcscmp( name, pResource->GetResourceName() );

  // if the strings match, we found the resource
  if( 0 == compare_result )
    return pResource;

  // now we need to figure out if there are more potential candidates or if we should just bail
  if( span_max == span_min )
    return NULL;

  if( compare_result < 0 )
  {
    if( index > span_min )
      return BinarySearchResources<T>( name, table, span_min, index-1);
  }
  else
  {
    if( index < span_max )
      return BinarySearchResources<T>( name, table, index+1, span_max);
  }
  return NULL;
}


#define FindResourceFunction(Type, type)                                                       \
    UWBD3D_##Type##Resource*                                                                  \
    UWBD3D_GraphicsSystem::Find##Type##Resource( const wchar_t* name, bool add_on_failure )       \
    {                                                                                        \
        if( wcslen(name) == 0 )                                                               \
           return NULL;                                                                       \
        UWBD3D_##Type##Resource*                                                               \
        pResource = BinarySearchResources<UWBD3D_##Type##Resource>(name, m_##type##_resources,     \
                                  0, m_##type##_resources.Count()-1 );                          \
        if( pResource )                                                                        \
           return pResource;                                                                   \
        if( add_on_failure ) {                                                                 \
           UWBD3D_##Type##Resource *pEntry = new UWBD3D_##Type##Resource();                       \
           if( !pEntry->LoadResource( m_pD3DDevice, name ) )  {                                  \
               delete pEntry;                                                                  \
               return NULL;                                                                    \
           }                                                                                   \
           for( int i=0; i<m_##type##_resources.Count(); i++ ) {                                 \
               UWBD3D_##Type##Resource* pResource = m_##type##_resources[i];                      \
               int compare_result = ::wcscmp( name, pResource->GetResourceName() );               \
               if( compare_result < 0 )   {                                                     \
                   m_##type##_resources.Insert( pEntry, i );                                     \
                   return pEntry;                                                              \
               }                                                                              \
           }                                                                                  \
           m_##type##_resources.Append( pEntry );                                               \
           return pEntry;                                                                      \
        }                                                                                     \
        return NULL;                                                                           \
    }

  FindResourceFunction(Texture, texture)
  FindResourceFunction(Mesh, mesh)
  FindResourceFunction(Effect, effect)




UWBD3D_MeshResource* UWBD3D_GraphicsSystem::GetMeshResource(const wchar_t* meshname)
{
  return FindMeshResource(meshname, true);
}
