#include "uwbgl_D3DPrimitiveMesh3.h"
#include "uwbgl_D3DGraphicsSystem7.h"

UWBD3D_PrimitiveMesh::UWBD3D_PrimitiveMesh(const wchar_t* resource_filename)
: m_resource_filename( resource_filename )
{
  UWBD3D_GraphicsSystem& graphics = UWBD3D_GraphicsSystem::GetSystem();
  UWBD3D_MeshResource* pMesh = graphics.GetMeshResource( m_resource_filename.c_str() );
  if ( pMesh )
    SetMaterial(pMesh->GetFirstSubMeshMaterial());
}

void UWBD3D_PrimitiveMesh::DrawPrimitive( eLevelOfDetail lod, UWB_DrawHelper& draw_helper )
{
  UWBD3D_GraphicsSystem& graphics = UWBD3D_GraphicsSystem::GetSystem();
  UWBD3D_MeshResource* pMesh = graphics.GetMeshResource( m_resource_filename.c_str() );
  if( pMesh ) {
        graphics.SetEffectParameters( *this );
        pMesh->DrawMesh(m_bTexturingEnabled, (!IsLightingEnabled()), m_FlatColor );
  }
}

void UWBD3D_PrimitiveMesh::SetupDrawAttributes(UWB_DrawHelper& draw_helper)
{
  __super::SetupDrawAttributes(draw_helper);
  draw_helper.SetColor2(m_ShadingColor);
  draw_helper.SetShadeMode(m_ShadeMode);
  draw_helper.SetFillMode(m_FillMode);

  UWBD3D_GraphicsSystem& graphics = UWBD3D_GraphicsSystem::GetSystem();
  UWBD3D_MeshResource* pMesh = graphics.GetMeshResource( m_resource_filename.c_str() );
  if ( pMesh )
    pMesh->SetFirstSubMeshMaterial( GetMaterial() );
}

const UWB_BoundingVolume* UWBD3D_PrimitiveMesh::GetBoundingVolume( eLevelOfDetail lod ) const
{
  UWBD3D_GraphicsSystem& graphics = UWBD3D_GraphicsSystem::GetSystem();
  UWBD3D_MeshResource* pMesh = graphics.GetMeshResource( m_resource_filename.c_str() );
  if ( pMesh )
    return pMesh->GetBoundingBox();

  return NULL;
}
