#include "uwbgl_D3DResource1.h"
#include "uwbgl_Utility2.h"

UWBD3D_Resource::UWBD3D_Resource()
: m_pDevice( NULL )
, m_ResourceName(L"")
{
}

UWBD3D_Resource::~UWBD3D_Resource() 
{
  ReleaseMemory();
}

void UWBD3D_Resource::ReleaseMemory()
{
  if( NULL != m_pDevice ) 
  {
    m_pDevice->Release();
    m_pDevice = NULL;
  }
  
  m_ResourceName = L"";
}


// Load the mesh (in D3D .x format) and store into the buffer of the device
bool UWBD3D_Resource::LoadResource( LPDIRECT3DDEVICE9 pDevice,
                                    const wchar_t* fileName )
{
  ReleaseMemory();
  m_pDevice = pDevice;
  if( NULL == m_pDevice )
    return false;

  m_pDevice->AddRef();

  m_ResourceName = fileName;

  std::wstring fullPath;
  FindFilePath(fileName, fullPath);
  std::wstring msg(L"**ERROR** [");

  if ( fullPath.empty() ) 
  {
    msg.append(fileName);
    msg.append(L"]: file not found!");
    UWB_EchoToStatusArea(msg.c_str());
    return false;
  }

  if (!LoadResourceFromFile( fullPath.c_str() ))
  {
    msg.append(fileName);
    msg.append(L"]: open for loading failed!");
    UWB_EchoToStatusArea(msg.c_str());
    return false;
  }

  return true;
}


const wchar_t* UWBD3D_Resource::GetResourceName() const
{
  return m_ResourceName.c_str();
}



// ========================  Support loading of resources.

// Potential Search Paths
// NULL terminated list of potential directories where
// texture and mesh resources may be located
static const wchar_t *sSearchPaths[] =
{
  L"",
  L"./Textures",
  L"Textures",
  L"./MeshFiles",
  L"MeshFiles",
  L"./Effects",
  L"Effects",
  L""
};

// given a file name, try to locate it in one of the search directories
// return the directory that the file is in on success
// return empty string
void UWBD3D_Resource::FindFilePath(const wchar_t* filename, std::wstring& full_path)
{
  full_path.clear();

  std::wstring tryPath(filename);    // start with the file name, if full path ...

  int i = 0;
  do
  {
    FILE* stream = NULL;
    if( 0 == ::_wfopen_s( &stream, tryPath.c_str(), L"r" ) )
    {
      full_path = tryPath;
      fclose(stream);
    } 
    i++;
    tryPath = sSearchPaths[i];
    tryPath.append(L"/");
    tryPath.append(filename);
  } while ( full_path.empty() && ( wcslen(sSearchPaths[i])>0 ) );
 
}